% demo code for MCCA

disp('Random phase-shifted surrogate data');

clear
close all
% check for NoiseTools
if 2 ~= exist('nt_mcca')
    error('Download NoiseTools from http://audition.ens.fr/adc/NoiseTools/ and put on path');
end

%{
The synthetic dataset consists of 10 data matrices, each with 10 channels.
%}

%{
Each data matrix is obtained by mixing 10 independent Gaussian noise sources
(different for each data matrix). The noise power is in 1/f^2 (obtained by
applying cumsum() to white noise --> random walk).

No target.
%}

nsamples=100000;
nchans=10;
nsets=10;

% dataset is structured as a 3D matrix
dataset=zeros(nsamples,nchans,nsets);
for iSet=1:nsets
    noise=randn(nsamples,nchans); % 
    noise=cumsum(nt_demean(noise));
    dataset(:,:,iSet)=noise;
end

% original data
figure(1); clf
nt_banner('original data')
subplot 121
x=dataset(:,:); % concatenate channelwise
x=nt_demean(x);
C=x'*x;
[A,score,AA]=nt_mcca(C,nchans);
z=x*A;
plot(score, '.-');
ylabel('variance'); xlabel('SC');
title('variance profile');
subplot 122
plot(z(:,1)); title('SC1');

% surrogate phase-shifted data
figure(2); 
nt_banner('surrogate, random phase-shift');
for iRepeat=1:10;
    for iSet=1:nsets
        dataset(:,:,iSet)=phaserandomized(dataset(:,:,iSet));
    end
    x=dataset(:,:); % concatenate channelwise
    x=nt_demean(x);
    C=x'*x;
    [A,score,AA]=nt_mcca(C,nchans);
    z=x*A;
    subplot 121
    plot(score, '.-');
    ylabel('variance'); xlabel('SC'); 
    title('variance profile');
    hold on
    subplot 122
    plot(z(:,1));
    title('SC 1');
    hold on
end
% Fig2: random phase shifting of each dataset should destroy any trace of a
% real shared component.  Nonetheless, each analysis shows much the same
% variance profile, and a clearly-shaped first SC component.  

% borrowed from https://www.parralab.org/isc/:
% -------------------------------------------------------------------------
function Xr = phaserandomized(X);
% Generate phase randomized surrogate data Xr that preserves spatial and
% temporal correlation in X, following Prichard D, Theiler J. Generating 
% surrogate data for time series with several simultaneously measured 
% variables. Physical review letters. 1994 Aug 15;73(7):951.

[T,D,N] = size(X);

Tr = round(T/2)*2; % this code only works if T is even; make it so
for i = 1:N
    Xfft = fft(X(:,:,i),Tr); % will add a zero at the end if uneven length
    Amp = abs  (Xfft(1:Tr/2+1,:)); % original amplitude
    Phi = angle(Xfft(1:Tr/2+1,:)); % orignal phase
    Phir = 4*acos(0)*rand(Tr/2-1,1)-2*acos(0); % random phase to add
    tmp(2:Tr/2,:) = Amp(2:Tr/2,:).*exp(sqrt(-1)*(Phi(2:Tr/2,:)+repmat(Phir,1,D))); % Theiler's magic
    tmp = ifft([Xfft(1,:); tmp(2:Tr/2,:); Xfft(Tr/2+1,:); conj(tmp(Tr/2:-1:2,:))]); % resynthsized keeping it real
    Xr(:,:,i) = tmp(1:T,:,:); % grab only the original length
end
end