% demo code for MCCA

disp('Phase shifted surrogate data, real target');

clear
% check for NoiseTools
if 2 ~= exist('nt_mcca')
    error('Download NoiseTools from http://audition.ens.fr/adc/NoiseTools/ and put on path');
end

%{
The synthetic dataset consists of 10 data matrices, each with 10 channels.
%}

%{
Each data matrix is obtained by mixing a sinusoidal target (same for each data matrix) 
with 9 independent Gaussian noise sources
(different for each data matrix). The noise power is in 1/f^2 (obtained by
applying cumsum() to white noise --> random walk).
%}

nsamples=100000;
nchans=10;
nsets=10;
SNR=10^-1;

f=10; 
target=sin(2*pi*f*(1:nsamples)'/nsamples);
target=nt_normcol(target);
target(1:round(nsamples/f*3))=0; target=flipud(target);
target(1:round(nsamples/f*3))=0; 

% original data
figure(1); clf
nt_banner('original data');
dataset=zeros(nsamples,nchans,nsets);
for iSet=1:nsets
    noise=cumsum(nt_demean(randn(nsamples,nchans)));
    n=noise*randn(nchans,nchans);
    t=target*randn(1,nchans);
    dataset(:,:,iSet)=n/rms(n(:))+sqrt(SNR)*t/rms(t(:));
end
% apply MCCA
x=dataset(:,:); % concatenate channelwise
x=nt_demean(x);
C=x'*x;
[A,score,AA]=nt_mcca(C,nchans);
z=x*A;
subplot 121;
plot(score, '.-'); title('1/f^2 noise, SNR=10^-2');
ylabel('variance'); xlabel('SC'); title('variance profile')
subplot 122; 
plot(z(:,1)); title('SC 1')


% phase-shifted surrogate data
figure(2); clf
nt_banner('surrogate, random phase-shift');
for iRepeat=1:10
    for iSet=1:nsets
        dataset(:,:,iSet)=phaserandomized(dataset(:,:,iSet));
    end
    % apply MCCA
    x=dataset(:,:); % concatenate channelwise
    x=nt_demean(x);
    C=x'*x;
    [A,score,AA]=nt_mcca(C,nchans);
    z=x*A;
    subplot 121;
    plot(score, '.-'); title('1/f^2 noise, SNR=10^-2');
    ylabel('variance'); xlabel('SC'); title('variance profile')
    hold on
    subplot 122; 
    plot(z(:,1)); title('SC 1')
    hold on
end
%Fig2: phase randomization destroys the temporal pattern found by MCCA (SC1) in the
%original data, suggesting that that pattern was real.


% borrowed from https://www.parralab.org/isc/:
% -------------------------------------------------------------------------
function Xr = phaserandomized(X);
% Generate phase randomized surrogate data Xr that preserves spatial and
% temporal correlation in X, following Prichard D, Theiler J. Generating 
% surrogate data for time series with several simultaneously measured 
% variables. Physical review letters. 1994 Aug 15;73(7):951.

[T,D,N] = size(X);

Tr = round(T/2)*2; % this code only works if T is even; make it so
for i = 1:N
    Xfft = fft(X(:,:,i),Tr); % will add a zero at the end if uneven length
    Amp = abs  (Xfft(1:Tr/2+1,:)); % original amplitude
    Phi = angle(Xfft(1:Tr/2+1,:)); % orignal phase
    Phir = 4*acos(0)*rand(Tr/2-1,1)-2*acos(0); % random phase to add
    tmp(2:Tr/2,:) = Amp(2:Tr/2,:).*exp(sqrt(-1)*(Phi(2:Tr/2,:)+repmat(Phir,1,D))); % Theiler's magic
    tmp = ifft([Xfft(1,:); tmp(2:Tr/2,:); Xfft(Tr/2+1,:); conj(tmp(Tr/2:-1:2,:))]); % resynthsized keeping it real
    Xr(:,:,i) = tmp(1:T,:,:); % grab only the original length
end
end

